require 'fake-intrinsics'

vobj = require( 'va' ).vobj

vspec = require( 'saotrace.raygen.ea').vspec

describe( 'annulus', function ()

    local spec = vspec.annulus

    local ok, args

    it( 'missing', function ()

        ok, args = vobj:validate( spec, 'a1')

        assert.is_false( ok )
        assert.matches( 'missing', args )
     end)


    it( 'illegal ri', function ()

        ok, args = vobj:validate( spec,
				 'a1',
				 {  ri = -1, ro = 1 }
			      )

	assert.is_false( ok )
	assert.matches( 'did not match type', args )

     end)

    it( "ri, ro", function ()

        ok, args = vobj:validate( spec,
				 'a1',
				 {  ri = 0.5,
				    ro = 1,
				 }
			      )

	assert.is_true( ok )
	assert.equals( 1, args.opts.ro )
	assert.equals( 0.5, args.opts.ri )
	assert.equals( 0, args.opts.x )
	assert.equals( 0, args.opts.y )
	assert.equals( 0, args.opts.z )
	assert.equals( 'random', args.opts.raydist.type )
     end)

    it( "ri, ro, x, y, z", function ()
        ok, args = vobj:validate( spec,
				 'a1',
				 {  ri = 0.5,
				    ro = 1,
				    x = 3,
				    y = 4,
				    z = 5
				 }
			      )

	assert.is_true( ok )
	assert.equals( 1, args.opts.ro )
	assert.equals( 0.5, args.opts.ri )
	assert.equals( 3, args.opts.x )
	assert.equals( 4, args.opts.y )
	assert.equals( 5, args.opts.z )

     end)

end)

describe( "circle", function ()

   local spec = vspec.circle

   local ok, args

    it( 'missing', function ()

        ok, args = vobj:validate( spec,
				 'a1'
			      )

	assert.is_false( ok )
	assert.matches( 'missing', args )

     end)

    it( 'r negative', function ()

        ok, args = vobj:validate( spec,
				 'a1',
				 {  r = -1 }
			      )

	assert.is_false( ok )
	assert.matches( 'did not match type', args )

     end)

    it( 'r zero', function ()
        ok, args = vobj:validate( spec,
				 'a1',
				 {  r = 0 }
			      )

	assert.is_false( ok )
	assert.matches( 'did not match type', args )
     end)

    it( 'r valid', function ()
        ok, args = vobj:validate( spec,
				 'a1',
				 {  r = 1,
				 }
			      )

	assert.is_true( ok )
	assert.equals( 1, args.opts.r )
	assert.equals( 0, args.opts.x )
	assert.equals( 0, args.opts.y )
	assert.equals( 0, args.opts.z )
     end)

    it( "r, x, y, z" , function ()

        ok, args = vobj:validate( spec,
				 'a1',
				 {  r = 1,
				    x = 3,
				    y = 4,
				    z = 5
				 }
			      )

	assert.is_true( ok )
	assert.equals( 1, args.opts.r )
	assert.equals( 3, args.opts.x )
	assert.equals( 4, args.opts.y )
	assert.equals( 5, args.opts.z )
     end)

end)

describe( "pinhole", function ()

   local ok, args

   local spec = vspec.pinhole

   it( "negative area", function ()

       ok, args = vobj:validate( spec, 'a1', {  area = -1 })

       assert.is_false( ok )
       assert.matches( 'did not match type', args )
    end)

   it( "zero area", function ()
       ok, args = vobj:validate( spec, 'a1', {  area = 0 })

       assert.is_false( ok, args )
       assert.matches( 'did not match type', args )
    end)

   it( "no area", function ()
       ok, args = vobj:validate( spec, 'a1' )

       assert.is_true( ok, args )
       assert.equals( 1, args.opts.area )
       assert.equals( 0, args.opts.x )
       assert.equals( 0, args.opts.y )
       assert.equals( 0, args.opts.z )
    end)

   it( "positive area", function ()
       ok, args = vobj:validate( spec, 'a1', {  area = 2 } )

       assert.is_true( ok, args )
       assert.equals( 2, args.opts.area )
       assert.equals( 0, args.opts.x )
       assert.equals( 0, args.opts.y )
       assert.equals( 0, args.opts.z )
    end)

   it( "area, x, y, z", function ()
       ok, args = vobj:validate( spec, 'a1',
			    {  area = 0.5, x = 3, y = 4, z = 5})

       assert.is_true( ok, args )
       assert.equals( 0.5, args.opts.area )
       assert.equals( 3, args.opts.x )
       assert.equals( 4, args.opts.y )
       assert.equals( 5, args.opts.z )
    end)
end)
