require 'fake-intrinsics'

local raygen = require( 'saotrace.raygen' )
local extent = require( 'saotrace.raygen.extent' )
local units  = require( 'saotrace.raygen.units' )
local convert = units.convert

local pos = { theta = { 20, 'arcmin' }, phi = { 30, 'degree' } , z = 390 }

describe( "disk", function ()


    local obj

    it( "valid specificaton", function ()

      assert.no.errors(
	  function()
	     obj = extent.disk{ name = 'a',
				 radius = { 10, 'arcsec' },
				 position = pos,
				 idist = { 'gaussian', sigma = { 22, 'arcsec' } }
			      }
	  end
       )

       local args = obj.args
       assert.equals( 'a', args.name )
       assert.equals( 10, convert( args.radius, nil, units.angular.arcsecond)[1] )
       assert.equals( 20, convert( args.position.theta, nil, units.angular.arcminute)[1] )
       assert.near( 30, convert( args.position.phi, nil, units.angular.degree)[1], 1e-6 )
       assert.equals( 390, args.position.z[1] )
       assert.near( 22, convert( args.idist.sigma, nil, units.angular.arcsecond)[1], 1e-6 )

    end)

    it( "bogus idist", function ()

        assert.error_matches(
           function()
	      obj = extent.disk{ name = 'a',
				  radius = { 10, 'arcsec' },
				  position = pos,
				  idist = { 'furity' }
			       }
	   end,
	   'unknown intensity distribution'
	)

    end)

 end)

describe( "rect", function ()

   local obj

    it( "valid specificaton", function ()

	assert.no.errors(
	   function ()
	      obj = extent.rect{ name = 'c',
			       height = { 10, 'arcsec' },
			       width  = { 11, 'arcsec' },
			       position = pos,
			       idist = { 'gaussian', sigma = { 22, 'arcsec' } }
			    }
	   end
	)

       local args = obj.args
	assert.equals( 'c', args.name )
	assert.equals( 10, convert( args.height, nil, units.angular.arcsecond)[1] )
	assert.equals( 11, convert( args.width, nil, units.angular.arcsecond)[1] )
	assert.equals( 20, convert( args.position.theta, nil, units.angular.arcminute)[1] )
	assert.near( 30, convert( args.position.phi, nil, units.angular.degree)[1], 1e-6 )
	assert.equals( 390, args.position.z[1] )
	assert.near( 22, convert( args.idist.sigma, nil, units.angular.arcsecond)[1], 1e-6 )

     end)

    it( "bogus idist", function ()

        assert.error_matches(
           function()
	      extent.rect{ name = 'a',
			    height = { 10, 'arcsec' },
			    width  = { 20, 'arcsec' },
			    position = pos,
			    idist = { 'furity'},
			 }
	   end,
	   'unknown intensity distribution'
	)

     end)
end)

describe( "point", function ()

    local obj
    it( "valid specificaton", function ()

      assert.no.errors(
	  function()
	     obj = extent.point{ name = 'b',
				  position = pos
			      }
	  end
       )

	local args = obj.args
	assert.equals( 'b', args.name )
	assert.equals( 20, convert( args.position.theta, nil, units.angular.arcminute)[1] )
	assert.near( 30, convert( args.position.phi, nil, units.angular.degree)[1], 1e-6 )
	assert.equals( 390, args.position.z[1] )

     end)

end)

describe( "image_pixs", function ()

    local obj
    it( "name, file, pixsz = abs", function ()

      assert.no.errors(
	  function()

	      obj = extent.image{
		 name = 'a',
		 file = 'foo',
		 pixsz = 0.1,
	      }
	   end
       )

	local args = obj.args
	assert.equals( 'a', args.name )
	assert.near( 0.1, convert(args.pixsz, nil, units.angular.arcsecond)[1], 1e-6 )

     end)

    it( "name, file, pixsz = arcmin", function ()

	assert.no.errors(
	    function()

		obj = extent.image{
		   name = 'a',
		   file = 'foo',
		   pixsz = { 10, 'arcminute' } ,
		}
	     end
	 )

	local args = obj.args
	assert.equals( 10, convert(args.pixsz, nil, units.angular.arcminute)[1], 1e-6 )

     end)


    it( "name, file, pixsz = (x,y) ", function ()

	assert.no.errors(
	    function()
	       obj = extent.image{
		  name = 'a',
		  file = 'foo',
		  pixsz = { x = 10, y = 20 }
	       }
	    end
	 )

	local args = obj.args
	assert.near( 10, convert(args.pixsz.x, nil, units.angular.arcsecond)[1], 1e-6 )
	assert.near( 20, convert(args.pixsz.y, nil, units.angular.arcsecond)[1], 1e-6 )

     end)

end)

