vobj = require('va').vobj

convert_angle = require( 'saotrace.raygen.units' ).convert_angle
idist_validate = require( 'saotrace.raygen.validate.idist' ).validate

function validate( ... )
   return idist_validate( vobj, ... )
end

describe( 'validate uniform1D', function ()

   it( 'legal', function()

       local ok, args = validate( {
				    x = { 'uniform' },
				    y = { 'uniform' },
				 })
       assert.is_true(ok, args)
       assert.is_equal( 'uniform', args.x[1] );
       assert.is_equal( 'uniform', args.y[1] );
    end  )

   it( 'extra args', function()

       local ok, args = validate( {
				    x = { 'uniform', 2 },
				    y = { 'uniform', 3 },
				 })
       assert.is_false(ok, args)
    end)

end)

describe( "validate gaussian1D", function ()

     it( 'missing args', function()

	 local ok, args = validate( {
					    x = { 'gaussian' },
					    y = { 'gaussian' },
					 })
	 assert.is_false(ok, args)

      end)

     it( 'mean', function ()

         local ok, args = validate( {
				      x = { 'gaussian', sigma = { 2, 'arcmin' } },
				      y = { 'gaussian', sigma = '3 arcmin' },
				   })
	 assert.is_true(ok, args)
	 assert.is_equal( 0, args.x.mean[1] );
	 assert.is_equal( 2, args.x.sigma[1] );

	 assert.is_equal( 0, args.y.mean[1] );

	 -- args.y is in units of 3 arcmin (yes!)
	 assert.is_equal( 1, args.y.sigma[1] );
	 local ok, val = convert_angle( 1, args.y.sigma[2], 'arcminute' )
	 assert.is_equal( 3, val[1], .01 )
      end)


     it( 'mean + sigma', function ()

         local ok, args = validate( {
				      x = { 'gaussian', sigma = '2 arcmin', mean = '1 arcmin' },
				      y = { 'gaussian', sigma = {4, 'arcmin'}, mean = {2, 'arcmin'} },
				   })
	 assert.is_true(ok, args)
	 assert.is_equal( 1, args.x.mean[1] );

	 -- args.x is in units of 2 arcmin
	 assert.is_equal( 1, args.x.sigma[1] );
	 local ok, val = convert_angle( 1, args.x.sigma[2], 'arcminute' )
	 assert.is_equal( 2, val[1], .01 )

	 assert.is_equal( 2, args.y.mean[1] );
	 assert.is_equal( 4, args.y.sigma[1] );
      end)

  end)

describe( "uniform2D", function ()

   it( 'legal', function()
       local ok, args = validate( { 'uniform' }  )
       assert.is_true(ok, args)
    end)

   it( 'extra args', function ()
       local ok, args = validate( { 'uniform', 3 }  )
       assert.is_false(ok, args)
    end)

end)

describe( "gaussian2D", function ()

    it( 'too few args', function ()
       local ok, args = validate( { 'gaussian' }  )
       assert.is_false(ok, args)
    end)

    it( 'symmetric sigma', function ()
	local ok, args = validate( { 'gaussian', sigma = { 2, 'arcsec' } }  )
	assert.is_true(ok, args)
	assert.is_equal( 0, args.x[1] );
	assert.is_equal( 0, args.y[1] );
	assert.is_equal( 2, args.sigma[1] );
     end)

    it( 'x + y + symmetric sigma', function ()

        local ok, args = validate( { 'gaussian', x = { 2, 'arcsec' },
				     y = '3 arcsec',
				     sigma = { 22, 'arcsec' } }  )
	assert.is_true(ok, args)
	assert.is_equal( 2, args.x[1] );

	-- args.y is in units of 3 arcsec (yes!)
	assert.is_equal( 1, args.y[1] );
	local ok, val = convert_angle( 1, args.y[2], 'arcsecond' )
	assert.near( 3, val[1], .01 )
	assert.is_equal( 22, args.sigma[1] );
     end)

end)
