require 'fake-intrinsics'

local extent = require( 'saotrace.raygen.extent' )
local spectrum = require( 'saotrace.raygen.spectrum' )

create = require( 'saotrace.raygen.source').create

describe( "defaults", function ()

   it( "missing argument", function()
       assert.error_matches( function () create() end,
			    '%(name%): missing' )

    end)

   it( "argument", function ()
       local ok, src
       assert.has_no.error( function() src = create('test' ) end )

       assert.equal( 'test', src.name )

       local default

       default = extent.default{ name = src.name }

       assert.equals( default.type, src.extent.type )

       assert.same( default.args, src.extent.args )

       default = spectrum.default{ name = src.name }

       assert.equals( default.type, src.spectrum[1].type )

       assert.same( default.args, src.spectrum[1].args )

    end)
end)

describe( "extent", function ()

   it ( "point", function ()

       local pos = extent.point{ name = 'tp' }

       local src
       assert.no.error( function () src = create( 'ts', { extent = pos } ) end )

       assert.equals( 'ts', src.name )

       assert.same( pos, src.extent )

    end)
end)

describe( "spectrum", function ()

    it( "mono + flat", function ()

       local s1 = spectrum.mono{ name = 's1', energy = 2 }
       local s2 = spectrum.flat{ name = 's2', emin = 1, emax = 10 }

       local src
       assert.no.error( function () src = create( 'ts', { spectrum = { s1, s2 } } ) end )

       assert.equals( 'ts', src.name )

       assert.same( s1, src.spectrum[1] )
       assert.same( s2, src.spectrum[2] )

    end)

end)

