units = require 'saotrace.raygen.units'

local math = require( 'math' )
local pi = math.pi
local deg2rad = pi / 180
local rad2deg = 180 / pi


describe( "normalize value", function ()

    local system = units.system

    it ( "value + string unit, no default unit", function ()

	local v, u = units.normalize_value( { 1, 'degree' } )

	assert.equals( 1, v )
	assert.is_true( u:are_convertible( units.angular.degree ) )
	assert.equals( 1, u:get_converter(units.angular.degree):convert( v ) )

     end)

    it( "value + string unit, default unit", function ()


	local v, u = units.normalize_value( { 1, 'degree' }, units.linear.meter )

	assert.equals( 1, v )
	assert.is_true( u:are_convertible( units.angular.degree ) )
	assert.equals( 1, u:get_converter(units.angular.degree):convert( v ) )

     end)

    it( "value + ut_unit unit, no default unit", function ()

	local v, u = units.normalize_value( { 1, units.angular.degree } )

	assert.equals( 1, v )
	assert.is_true( u:are_convertible( units.angular.degree ) )
	assert.equals( 1, u:get_converter(units.angular.degree):convert( v ) )

     end)

    it( "value + ut_unit unit, no default unit", function ()

	local v, u = units.normalize_value( { 1, units.angular.degree }, units.linear.meter )

	assert.equals( 1, v )
	assert.is_true( u:are_convertible( units.angular.degree ) )
	assert.equals( 1, u:get_converter(units.angular.degree):convert( v ) )

     end)

    it( "value, no default unit", function ()

	local v, u = units.normalize_value( 1 )

	assert.equals( 1, v )
	assert.equals( nil, u )

     end)

    it( "value, default unit", function ()

	local v, u = units.normalize_value( 1, units.angular.degree )

	assert.equals( 1, v )
	assert.is_true( u:are_convertible( units.angular.degree ) )
	assert.equals( 1, u:get_converter(units.angular.degree):convert( v ) )
     end)

end)
