va = require( 'saotrace.raygen.validate' )

units = require 'saotrace.raygen.units'

local math = require( 'math' )
local pi = math.pi
local deg2rad = pi / 180
local rad2deg = 180 / pi

describe( "ra", function ()

    describe( "good", function ()

       for hms, comp in pairs {
	  ['10:20:.5'] = { 10, 20, 0.5 },
	  ['10:20:5'] = { 10, 20, 5 },
	  ['0:2:5'] = { 0, 2, 5 },
       } do

	  it( hms, function ()

	      local ok, angle

	      local degrees = 15 * ( comp[1] + ( comp[2] + comp[3] / 60 ) / 60 )
	      local radians = degrees * deg2rad

	      ok, angle = va.validate_angle_ra( hms )

	      assert.is_true( ok )
	      assert.near( radians, angle[1], 1e-12 )

	      ok, angle = va.validate_angle_ra( degrees )
	      assert.is_true( ok, angle )
	      assert.near( radians, angle[1], 1e-12 )

	      ok, angle = va.validate_angle_ra( { radians, 'radians' } )
	      assert.is_true( ok, angle )
	      assert.near( radians, angle[1], 1e-12 )

	   end)

       end

    end)

    describe( "bad", function ()

       for _,v in pairs{
	  '10:20',
	  '10:20:x',
	  '10:20:.',
       } do

	  local ok, angle

	  it( v, function ()

	      ok, angle= va.validate_angle_ra( v )
	      assert.is_false( ok, angle )

	   end)

       end

    end)


 end)

describe( "dec", function()

    describe( "good", function ()

       for hms, comp in pairs {
	  ['-10:20:.5'] = { -1, 10, 20, 0.5 },
	  ['+10:20:.5'] = {  1, 10, 20, 0.5 },
	  ['10:20:.5']  = {  1, 10, 20, 0.5 },
       } do

	  it( "hms", function ()

	      local degrees = comp[1] * ( comp[2] + ( comp[3] + comp[4] / 60 ) / 60 )
	      local radians = degrees * deg2rad

	      local ok, angle = va.validate_angle_dec( hms )

	      assert.is_true( ok, angle )
	      assert.near( radians, angle[1], 1e-12 )

	      local ok, angle = va.validate_angle_dec( degrees )
	      assert.is_true( ok, angle )
	      assert.near( radians, angle[1], 1e-12 )

	      local ok, angle = va.validate_angle_dec( { radians, 'radians' } )
	      assert.is_true( ok, angle )
	      assert.near( radians, angle[1], 1e-12 )

	   end)

       end

    end)

    describe( "bad", function ()

	for _,v in pairs{
	   '10:20',
	   '10:20:x',
	   '10:20:.',
	   '-90:1:1',
	   '90:1:1',
	} do

	   it( v, function()
	       local ok, angle = va.validate_angle_dec( v )
	       assert.is_false( ok, angle )
	    end)


	end

     end)

end)

describe( "posquant", function ()

    it( "posquant", function ()
	assert.is_true( va.posquant( { 3 } ) )
	assert.is_false( va.posquant( { -3 } ) )
	assert.is_false( va.posquant( { 0 } ) )

	assert.is_true( va.zposquant( { 3 } ) )
	assert.is_false( va.zposquant( { -3 } ) )
	assert.is_true( va.zposquant( { 0 } ) )
     end)

end)
