#ifndef JITTER_LJ_H
#define JITTER_LJ_H

#include <string>
#include <cmath>

#include "Jitter/Base.h"

namespace Jitter {

    class LJ: public Base {

    public:

	struct Param {
	    double phase, amp, period;

	    Param( double phase, double amp, double period ) :
	    phase( phase ), amp( amp ), period ( period ) {}

	    Param() {}

	    double calc( double time ) {
		return amp * sin( time * 2 * M_PI / period  + phase );
	    }

	};

        LJ( std::string& name, Param &pitch, Param &yaw ) : Base(name), _pitch( pitch ), _yaw( yaw ), _last( 1, 0, 0, 0 ) {}
        LJ( const char* name,  Param &pitch, Param &yaw ) : Base(name), _pitch( pitch ), _yaw( yaw ), _last( 1, 0, 0, 0 ) {}
	~LJ() {}

	void jitter( double time, Eigen::Quaterniond &q ) {

	    Eigen::Quaterniond now = Eigen::Quaterniond( Eigen::AngleAxisd( _yaw.calc(time),   Eigen::Vector3d( 0, 1, 0 ) ) )
		                   * Eigen::Quaterniond( Eigen::AngleAxisd( _pitch.calc(time), Eigen::Vector3d( 1, 0, 0 ) ) );

	    q = now * _last.inverse();
	    _last = now;
	}

	Eigen::Quaterniond last() { return _last; }
	Param pitch() { return _pitch; }
	Param yaw() { return _yaw; }

    private:

	Param _pitch;
	Param _yaw;
	Eigen::Quaterniond _last;

    };

}


#endif // ! JITTER_LJ_H
