// -*-C++-*-
#ifndef SIMPLE_FITS_H
#define SIMPLE_FITS_H

#include <string>
#include <map>

#include <fitsio.h>

#include "Exception/Exception.h"

namespace SimpleFits {

    class File {

    public:

	fitsfile *fptr;

        File( ) : fptr( NULL), status( 0 ) {};
	File( const string& file );
	~File( void );

        bool read_keyword( const char*, double& value);
	double read_keyword_double( const char* keyname ) throw( Exception );
	double read_keyword_double( string& keyname ) throw( Exception ) {
	    return read_keyword_double( keyname.c_str() );
	}

        string read_keyword( const char* keyname ) throw( Exception );
	string read_keyword( const string& keyname ) throw( Exception ) {
	    return read_keyword( keyname.c_str() );
	}

    protected:
	int status;
	string    file;
	Exception exception( string pfx = "" );
    };

    //

    class Table : public File {

    public:

	struct Column {

	    int      typecode;
	    int      colnum;
	    string   name;
	    LONGLONG repeat;
	    string   unit;
	    long     width;

	    Column() {};
	    Column( Table& tbl, const string& alias, const string& name );

	};

	long num_rows( void ) { return _nrows; }
	long rowsize( void )  { return _rowsize; }
	LONGLONG read_col_double( double* &dst, const Column& col, LONGLONG start_row, LONGLONG nrows );


	Column column( const string& alias, const string& colname ) {

	    return Column( *this, alias, colname );
	}

	bool exists_column( const string& column_name );

	Table( const string& file );
    private:

	long _nrows;
	long _rowsize;
    };

    ostream& operator<<( std::ostream& ostr, const Table::Column& c );

}

#endif // ! FITS_FILE_H
