#ifndef DIST_H_included
#define DIST_H_included

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <udunits2.h>

typedef enum
{
  DT_NONE,
  DT_UNIFORM,
  DT_GAUSSIAN
} DistType;

typedef void(*gen2D)(void *data, double *x, double *y );
typedef void(*gen1D)(void *data, double *x );


typedef struct {

    double covm[4];
    double meanv[2]; 
    double gparm[2*(2+3)/2 + 1 ];

} Gauss2D;

typedef struct {
    double sigma;
    double mean;

} Gauss1D;

typedef struct
{
    DistType	type;		/* distribution type */

    double min, max;
    gen1D  gen;

    union {
	Gauss1D gauss;
    } dist;
} Dist1D;

typedef struct
{

    DistType type;

    double xmin, xmax;
    double ymin, ymax;
    gen2D  gen;

    union {
	Gauss2D gauss;
    } dist;

} Dist2D;


typedef struct
{
    int twoD;

    union {
	struct { Dist1D x, y; } x_y;
	Dist2D xy;
    } dist;

} ImgDist;


void get_ImgDist( lua_State *L, const char *err_pfx, ImgDist *idist, double z,
		  ut_unit* unit,
		  double xmin, double xmax, double ymin, double ymax );


void gen_ImgDist( ImgDist *idist, double *x, double *y );

#endif /* ! DIST_H_included */
