/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2010 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>

#include <math.h>

#include <lua.hpp>
#include <lauxlib.h>

#include <mathconst/mathconst.h>
#include <randlib_c/randlib.h>

#include "main.h"
#include "lua_stack.h"
#include "ea.h"

static int	define(lua_State *);
static void	random_pos( void *data, DVector3 *pos );
static void     cleanup( void * data );

luaL_Reg ea_annulus = { "annulus", define };

typedef struct Annulus
{
    double ri, ro;
    double ri2;		/* square of the inner radius */
    double dr2;		/* difference of the squared inner and outer
			   radii */
    double area;
    DVector3 origin;
} Annulus;


static int
define( lua_State *L )
{
    Annulus *data;
    char err_pfx[1024];
    static char *name;

    name = get_lua_cstring( L, "name" );

    /* generate error message prefix */
    sprintf( err_pfx, "ea.annulus `%s'", name );

    if ( NULL == ( data = (Annulus *) malloc( sizeof(Annulus) ) ) )
	luaL_error( L, "%s: error allocating memory", err_pfx );

    /* move opts table to top of Lua stack */
    lua_getfield( L, -1, "opts" );

    data->origin.x = get_lua_number( L, "x" );
    data->origin.y = get_lua_number( L, "y" );
    data->origin.z = get_lua_number( L, "z" );
    data->ri       = get_lua_number( L, "ri" );
    data->ro       = get_lua_number( L, "ro" );

    data->ri2 = data->ri * data->ri;
    data->dr2 = data->ro * data->ro - data->ri2;

    data->area = M_PI * data->dr2 / 100;

    new_entrance_aperture( L, err_pfx,
			   data,
			   get_lua_number( L, "id" ),
			   data->area,
			   random_pos,
			   cleanup
			   );
    free( name );

    return 0;
}



static void
random_pos( void *data, DVector3 *pos )
{
  Annulus *ann = (Annulus *) data;
  double r = sqrt( ann->ri2 + ann->dr2 * ranf() );
  double phi = M_2PI * ranf();

  pos->x = r * cos( phi ) + ann->origin.x;
  pos->y = r * sin( phi ) + ann->origin.y;
  pos->z = ann->origin.z;
}


static void
cleanup( void * data )
{
  free( data );
}
