/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2010 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

#include <math.h>
#include <stdlib.h>

#include <lua.hpp>
#include <lauxlib.h>

#include <mathconst/mathconst.h>
#include <randlib_c/randlib.h>

#include "main.h"
#include "lua_stack.h"
#include "ea.h"

static int	define(lua_State *);
static void	random_pos( void *data, DVector3 *pos );
static void     cleanup( void * data );

luaL_Reg ea_circle = { "circle", define };

typedef struct Circle
{
  double r;
  double r2;
  double area;
  DVector3 origin;
} Circle;


static int
define( lua_State *L )
{
    Circle *data;
    char err_pfx[1024];
    static char *name;

    name = get_lua_cstring( L, "name" );

    /* generate error message prefix */
    sprintf( err_pfx, "ea.circle `%s'", name );

    if ( NULL == ( data = (Circle *) malloc( sizeof(Circle) ) ) )
	luaL_error( L, "%s: error allocating memory", err_pfx );

    /* move opts table to top of Lua stack */
    lua_getfield( L, -1, "opts" );

    data->origin.x = get_lua_number( L, "x" );
    data->origin.y = get_lua_number( L, "y" );
    data->origin.z = get_lua_number( L, "z" );
    data->r        = get_lua_number( L, "r" );

    data->r2 = data->r * data->r;
    data->area = M_PI * data->r2 / 100;

    new_entrance_aperture( L, err_pfx,
			   data,
			   get_lua_number( L, "id" ),
			   data->area,
			   random_pos,
			   cleanup
			   );
    free( name );

    return( 0 );
}


static void
random_pos( void *data, DVector3 *pos )
{
  Circle *circle = (Circle *) data;
  double r = sqrt( circle->r2 * ranf() );
  double phi = M_2PI * ranf();

  pos->x = r * cos( phi ) + circle->origin.x;
  pos->y = r * sin( phi ) + circle->origin.y;
  pos->z = circle->origin.z;
}

static void
cleanup( void *data )
{
  free( data );
}
