/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2010 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

#include <math.h>
#include <stdlib.h>

#include <lua.hpp>
#include <lauxlib.h>

#include <mathconst/mathconst.h>
#include <randlib_c/randlib.h>

#include "main.h"
#include "lua_stack.h"
#include "ea.h"

static int	define(lua_State *);
static void	pos( void *data, DVector3 *pos );
static void     cleanup( void * data );

luaL_Reg ea_pinhole = { "pinhole", define };

typedef struct Pinhole
{
  double area;
  DVector3 origin;
} Pinhole;


static int
define( lua_State *L )
{
    Pinhole *data;
    char err_pfx[1024];
    static char *name;

    name = get_lua_cstring( L, "name" );

    /* generate error message prefix */
    sprintf( err_pfx, "ea.pinhole `%s'", name );

    if ( NULL == ( data = (Pinhole *) malloc( sizeof(Pinhole) ) ) )
	luaL_error( L, "%s: error allocating memory", err_pfx );

    /* move opts table to top of Lua stack */
    lua_getfield( L, -1, "opts" );

    data->origin.x = get_lua_number( L, "x" );
    data->origin.y = get_lua_number( L, "y" );
    data->origin.z = get_lua_number( L, "z" );
    data->area     = get_lua_number( L, "area" );

    new_entrance_aperture( L,
			   err_pfx,
			   data,
			   get_lua_number( L, "id" ),
			   data->area,
			   pos,
			   cleanup
			   );
    free( name );

    return( 0 );
}


static void
pos( void *data, DVector3 *pos )
{
  Pinhole *pinhole = (Pinhole *) data;
  pos->x = pinhole->origin.x;
  pos->y = pinhole->origin.y;
  pos->z = pinhole->origin.z;
}

static void
cleanup( void *data )
{
  free( data );
}
