#ifndef EVENT_H
#define EVENT_H

typedef struct Event Event;

#include "flux_gen.h"
#include "pos_gen.h"
#include "flux_mod.h"

typedef enum
{
  Event_FLUX,
  Event_EXTENT
} EventType;

typedef void(*EventService)(void *module, double time, void *module_data, void *event_data);
typedef void(*EventDelete)(void *event);
typedef void(*EventCleanup)(struct Event *event);

struct Event
{
  double	time;		/* time of the event */
  EventType	type;		/* type of event */
  void *	module;		/* module that caused the event */
  void *	module_data;	/* module specific data */
  void *	event_data;	/* event specific data */
  EventService  service;	/* service routine */
  EventDelete   del;		/* delete event data */
  EventCleanup	cleanup;	/* cleanup routine */
};


void register_event(char *mod_name, EventType event_type, void *module,
		    double event_time, double current_time,
		    void *module_data, void *event_data,
		    EventService service,
		    EventDelete  del,
		    EventCleanup cleanup,
		    SLinkList queue);

void delete_event(void *event);

typedef void (*PosGenEventService)(void *gen, double time, PositionGenerator *module_data, void *event_data);
typedef void (*FluxGenEventService)(void *gen, double time, FluxGeneratorData *module_data, void *event_data);


void register_flux_gen_event(FluxGenerator *gen, double time, void *data,
			     FluxGenEventService service, EventDelete del );

void register_flux_modifier_event(FluxModifier *mod, double time, void *data,
				  EventService service, EventDelete del );

void register_extent_event(PositionGenerator *gen, double time, void *data,
			   PosGenEventService service, EventDelete del );

int event_cmp(const void *event1, const void *event2);

#endif /* ! EVENT_H */
