/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <lua.hpp>
#include <lauxlib.h>
#include <stdlib.h>

#include <randlib_c/randlib.h>

#include "main.h"
#include "lua_stack.h"
#include "flux_gen.h"

static int	define(lua_State *L);
static int	init(FluxGeneratorData *data);
static int	generate( void *data, double *energy );
static void	cleanup(void *data);

luaL_Reg spectrum_flat = { "flat", define };

typedef struct
{
  double	energy_min;	/* energy */
  double	energy_max;	/* energy */
  double	base_flux;	/* base flux in photons/s/cm^2 */
} FlatData;

static FluxGeneratorSpec spec =
{
  "flat",
  init,
  generate,
  cleanup
};

static int
define( lua_State *L )
{
  FlatData *data;
  char *name;
  char err_pfx[1024];

  name = get_lua_cstring( L, "name" );

  /* generate error message prefix */
  sprintf( err_pfx, "flat `%s'", name );

  if ( NULL == ( data = (FlatData *) malloc( sizeof(FlatData) ) ) )
      luaL_error( L, "%s: error allocating memory", err_pfx );

  data->energy_min = get_lua_number( L, "emin" );
  data->energy_max = get_lua_number( L, "emax" );
  data->base_flux  = get_lua_number( L, "flux" );

  if ( data->energy_min >= data->energy_max )
      luaL_error( L, "%s: energy max (%f) must be greater than energy min (%f)",
		  err_pfx, data->energy_max, data->energy_min );

  new_flux_gen( &spec, name, data );

  return 0;
}


 static int
init( FluxGeneratorData *data )
{
  data->base_flux = ((FlatData *) data->data)->base_flux;
  data->iter_lim = 100;
  data->iter_warn = 10;

  return 0;
}

static int
generate( void *data, double *energy )
{
  FlatData *flat =  (FlatData *) data;

  *energy = genunf( flat->energy_min, flat->energy_max );

  return 0;
}

static void
cleanup( void *data )
{
  free ( data );
}
