#ifndef FLUX_GEN_H
#define FLUX_GEN_H

typedef struct FluxGeneratorData FluxGeneratorData;
typedef struct FluxGenerator  FluxGenerator;

#include "source.h"
#include "events.h"

struct FluxGeneratorData
{
  double	base_flux;		/* base flux, in photons/s/cm^2 */
  size_t	iter_lim;		/* maximum number of iterations to
  					   satisfy modifier events */
  size_t	iter_warn;		/* number of iterations
					   between modifier warnings */
  void *	data;			/* flux generator specific data */
};

typedef int(*FluxGenInit)(FluxGeneratorData *data);
typedef int(*FluxGenGenerate)(void *data, double *energy);
typedef void(*FluxGenCleanup)(void *data);


typedef struct FluxGeneratorSpec
{
    const char *	    name;	/* the name of the flux generator */
    FluxGenInit     init;
    FluxGenGenerate generate;
    FluxGenCleanup  cleanup;
} FluxGeneratorSpec;

struct FluxGenerator
{
  struct Source *	source;			/* the source that this generator
  					   belongs to */

  size_t	id;			/* generator id.  used as offset
					   in source's fluxgen array */

  size_t	n_mods;			/* the number of modifiers */
  SLinkList 	mods;			/* list of modifiers */

  double *	flux_factors;		/* array of factors which
					   modify the base flux */
  double *	flux_factors_lim;	/* a pointer to one past the
					   array of flux factors */
  double	flux_factor;		/* product of all of the flux
					   factors */
  double	eff_flux;		/* effective flux */

  SLinkList 	events;			/* event queue for generator and
  					   modifiers */

  double	time;			/* the time of arrival of the next
  					   photon from this generator */
  FluxGeneratorSpec spec;
  FluxGeneratorData data;		/* generator specific data */
  char *	name;			/* the generator specific name */

};

void new_flux_gen(FluxGeneratorSpec *spec, char *name, void *data);
void delete_flux_gen(void *data);
int  init_flux_gen(void *data);
void calculate_eff_flux(FluxGenerator *gen);
void cleanup_flux_gen_event(Event *event);
int  set_flux_gen_time(void *data, void *udata);
int  flux_gen_time_cmp(const void *gen1, const void *gen2);

#endif /* ! FLUX_GEN_H */
