#ifndef FLUX_MOD_H
#define FLUX_MOD_H

#include "source.h"
#include "flux_gen.h"
#include "events.h"

typedef struct
{
  const char *	name;			/* the generic name of the flux
					   modifier */
  int		(*init)   ( void *data );
  void		(*cleanup)( void *);
} FluxModifierSpec;

typedef struct
{
  char *	name;			/* the specific label for this
					   modifier */
  struct FluxGenerator *gen;		/* the generator to modify */
  struct Source	*	source;		/* the parent source of the
					   generator */
  size_t	id;			/* modifier id */

  double *	factor;			/* a pointer to this modifier's
					   factor slot in the parent
					   generator's list. */
  FluxModifierSpec spec;		/* modifier specific specification */
  void *	data;			/* modifier specific data */
} FluxModifier;


void new_flux_modifier(FluxModifierSpec *spec, char *name, void *data);
void delete_flux_modifier(void *data);
int  init_flux_modifier(FluxModifier *mod);
void cleanup_flux_modifier_event(Event *event);

#endif /* !  FLUX_MOD_H */
