#ifndef IMAGE_H
#define IMAGE_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <lua.hpp>

#include <suplib/str.h>


typedef struct
{
  int		n;		/* number of pixels */
  double	pix_sz;		/* size of a pixel in radians */
  double	pix_delta;	/* size of a pixel in radians,
				   including sign */
  int		flip;		/* flip order of axis */

  double	ref_idx;	/* pixel index of reference point */
  double	ref_wcs;	/* WCS coordinate at reference point */
  double	pnt_wcs;	/* WCS coordinate corresponding to
				   angular position of source */
  double	pnt_idx;	/* pixel index corresponding to
				   angular position of source */
} Axis;

void rd_image_hdr_pxm(lua_State *L, const char *err_pfx, const char *image,
			   int *nx, int *ny);

void rd_image_hdr_fits(lua_State *L, const char *err_pfx, const char *image, double z,
		       Axis *x, Axis *y);

/* image reading stuff */
char   *rd_line( FILE *file, void *rec );
double *rd_image_data_fits( const char *err_pfx, const char *image );
double *rd_image_data_ascii( const char *err_pfx, const char* image, size_t npix );
double *rd_image_data_float( const char *err_pfx, const char *image, size_t npix);
double *rd_image_data_double( const char *err_pfx, const char *image, size_t npix);
double *rd_image_data_pxm( const char *err_pfx, const char *image );


#endif /* !IMAGE_H */
