/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *       Free Software Foundation, Inc.
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <math.h>
#include <string.h>
#include <string>

#include <lua.hpp>
#include <lauxlib.h>

#include <tracefctxx/TraceFct.h>
#include <tracefct/exiterrvals.h>

#include "main.h"
#include "common.h"

#include "lua_stack.h"
#include "misc.h"

#include "Jitter/LJ.h"

static Jitter::LJ::Param get_aparam( lua_State *, const char*, const char*);

static int	define(lua_State *);

luaL_Reg jitter_lissajous = { "lissajous", define };

static int
define( lua_State *L )
{
    TraceFct tf( "jitter.lissajous.define");

    char err_pfx[1024];

    std::string name = get_lua_string( L, "name" );

    /* generate error message prefix */
    sprintf( err_pfx, "jitter.lissajous `%s'", name.c_str() );

    Jitter::LJ::Param pitch = get_aparam( L, err_pfx, "pitch" );
    Jitter::LJ::Param yaw   = get_aparam( L, err_pfx, "yaw" );

    Jitter::LJ* lj = new Jitter::LJ( name, pitch, yaw );

    sys.jitter.generators.push_back( lj );

    return 0;
}

static Jitter::LJ::Param
get_aparam( lua_State *L, const char* err_pfx, const char* field )
{
    Jitter::LJ::Param aparam;

    lua_getfield(L, -1, field);

    aparam.phase  = get_lua_angular( L, err_pfx, "phase", 0 );
    aparam.amp    = get_lua_angular( L, err_pfx, "amp", 0 );
    aparam.period = get_lua_temporal( L, err_pfx, "period", 0 );

    lua_pop( L, 1 );

    return aparam;
}
