#include <iostream>

#include "lua_fits.h"

namespace LuaFits {

    const char File::className[] = "file";

    Lunar<File>::RegType File::methods[] = {

	LUNAR_DECLARE_METHOD( File, keyword_number ),
	LUNAR_DECLARE_METHOD( File, keyword ),
	{ 0, 0 }
    };

    File::File( lua_State* L )  {

	try {
	    const char* filename = luaL_checkstring( L, 1 );
	    file = new SimpleFits::File( filename );
	}
	catch ( Exception& e ) {

	    lua_pushstring( L, e.what() );
	    lua_error( L );
	}

    }

    int
    File::keyword_number( lua_State *L ) {
	const char *keyword = luaL_checkstring( L, 1 );
	lua_pushnumber( L, file->read_keyword_double( keyword ) );
	return 1;
    }

    int
    File::keyword( lua_State *L ) {
	const char* keyword = luaL_checkstring( L, 1 );
	string value;

	try {

	    value = file->read_keyword( keyword );

	}
	catch ( Exception& e ) {
	    lua_pushstring( L, e.what() );
	    lua_error( L );
	}

	lua_pushstring( L, value.c_str() );

	return 1;
    }

    const char Table::className[] = "table";

    Lunar<Table>::RegType Table::methods[] = {

	LUNAR_DECLARE_METHOD( Table, keyword_number ),
	LUNAR_DECLARE_METHOD( Table, keyword ),
	LUNAR_DECLARE_METHOD( Table, exists_column ),
	{ 0, 0 }
    };

    Table::Table( lua_State *L ) {

	try {
	    const char* filename = luaL_checkstring( L, 1 );
	    file = new SimpleFits::Table( filename );
	}
	catch ( Exception& e ) {

	    lua_pushstring( L, e.what() );
	    lua_error( L );
	}

    }

    int
    Table::keyword_number( lua_State *L ) {
	const char *keyword = luaL_checkstring( L, 1 );
	lua_pushnumber( L, file->read_keyword_double( keyword ) );
	return 1;
    }

    int
    Table::keyword( lua_State *L ) {
	const char* keyword = luaL_checkstring( L, 1 );
	string value;

	try {

	    value = file->read_keyword( keyword );

	}
	catch ( Exception& e ) {
	    lua_pushstring( L, e.what() );
	    lua_error( L );
	}

	lua_pushstring( L, value.c_str() );

	return 1;
    }


    int
    Table::exists_column( lua_State *L ) {

	const char* column = luaL_checkstring( L, 1 );

	bool exists = file->exists_column( column );

	lua_pushboolean( L, exists ? 1 : 0 );
	return 1;
    }

}
