#include <stddef.h>

/* for setenv */
#define _POSIX_C_SOURCE 200112L
#include <stdlib.h>
#undef _POSIX_C_SOURCE

#include <lua.hpp>
#include <lauxlib.h>

#include <suplib/str.h>
#include <tracefctxx/TraceFct.h>

#include "config.h"

#include "LuaState.h"

#include "lua_fits.h"
#include "lua_raygen.h"

/*
   Inserting a new Lua accessible C function
   -----------------------------------------
*/

/****************************************
 * 1. Declare luaL_Reg structures       *
 ****************************************/
extern luaL_Reg extent_disk;
extern luaL_Reg extent_image;
extern luaL_Reg extent_rect;
extern luaL_Reg extent_point;

extern luaL_Reg spectrum_flat;
extern luaL_Reg spectrum_mono;
extern luaL_Reg spectrum_file;

extern luaL_Reg ea_circle;
extern luaL_Reg ea_annulus;
extern luaL_Reg ea_wedge;
extern luaL_Reg ea_pinhole;

extern luaL_Reg src_start;
extern luaL_Reg src_finish;

extern luaL_Reg jitter_roll;
extern luaL_Reg jitter_file;
extern luaL_Reg jitter_lissajous;

extern luaL_Reg units_system;

/*************************************************************
 * 2. Add a pointer to the structure to the appropriate list *
 *************************************************************/

/*----------------------------------------------*/
/* Extents */

static
luaL_Reg *extent_funcs[] =  {
    &extent_disk,
    &extent_image,
    &extent_rect,
    &extent_point,

    /* -- place new luaL_Reg structures above this line  -- */
    NULL
};

/*----------------------------------------------*/
/* Spectrum */

static
luaL_Reg *spectrum_funcs[] =  {
    &spectrum_mono,
    &spectrum_flat,
    &spectrum_file,

    /* -- place new luaL_Reg structures above this line  -- */
    NULL
};

/*----------------------------------------------*/
/* Entrance aperture */

static
luaL_Reg *ea_funcs[] = {
    &ea_circle,
    &ea_annulus,
    &ea_wedge,
    &ea_pinhole,

    /* -- place new luaL_Reg structures above this line  -- */
    NULL
};

/*----------------------------------------------*/
/* Source */

static
luaL_Reg *src_funcs[] = {

    &src_start,
    &src_finish,

    /* -- place new luaL_Reg structures above this line  -- */
    NULL
};

/*----------------------------------------------*/
/* Jitter */

static
luaL_Reg *jitter_funcs[] = {

    &jitter_file,
    &jitter_lissajous,
    &jitter_roll,

    /* -- place new luaL_Reg structures above this line  -- */
    NULL
};

/*----------------------------------------------*/
/* Units */

static
luaL_Reg *units_funcs[] = {

    &units_system,

    /* -- place new luaL_Reg structures above this line  -- */
    NULL
};


/*************************************************************
 * 3. Don't touch anything below here                        *
 *************************************************************/

static int
version( lua_State *L ) {

    lua_pushstring( L, VERSION );
    return 1;
}



void
luaopen_raygen( LuaState& L )
{
    TraceFct tf( "luaopen_raygen" );

    /* load intrinsic functions */
    L.register_module( "saotrace.raygen.intrinsic.jitter",   jitter_funcs );
    L.register_module( "saotrace.raygen.intrinsic.extent",   extent_funcs );
    L.register_module( "saotrace.raygen.intrinsic.spectrum", spectrum_funcs );
    L.register_module( "saotrace.raygen.intrinsic.ea",       ea_funcs );
    L.register_module( "saotrace.raygen.intrinsic.source",   src_funcs );
    L.register_module( "saotrace.raygen.intrinsic.units",    units_funcs );

    // add FITS lua interface
    lua_newtable( L );
    Lunar<LuaFits::File>::Register(L);
    Lunar<LuaFits::Table>::Register(L);
    L.register_module( "saotrace.raygen.fits" );

    /* load raygen lua code. do this after loading the intrinsic functions
       to ensure that the Lua wrappers are created properly
     */
    L.require_module( "saotrace.raygen" );

    lua_pushcfunction(L, version );
    L.set_module_attr( "saotrace.raygen", "version" );

}

