#ifndef LUA_STACK_H
#define LUA_STACK_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *       Free Software Foundation, Inc.
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <string>

#include <lua.hpp>
#include <suplib/str.h>

#include <udunits2.h>

char *get_lua_cstring( lua_State *L, const char* name );

std::string  get_lua_string( lua_State *, const char* );
int          get_lua_string( lua_State *, const char*, const char*, std::string&  );

double get_lua_number( lua_State *, const char* );
int    get_lua_number( lua_State *, const char*, const char*, double& );

bool get_lua_boolean( lua_State *, const char* );
int  get_lua_boolean( lua_State *, const char*, const char*, bool& );

double get_lua_extent( lua_State *L,  const std::string& err_pfx,
		       double dist, ut_unit* tunit);

double get_lua_extent( lua_State *L,  const char *err_pfx,
		       const char *name,  double dist, ut_unit* tunit);

int get_lua_token ( lua_State *L, const char* name, TokList *tokens );

bool exists_lua_field( lua_State *L, const char* name );

double get_lua_temporal( lua_State* L, const char * err_pfx, const char *var, double defval );

double get_lua_linear( lua_State* L, const char * err_pfx, const char *var, double defval );

double get_lua_angular( lua_State* L, const char * err_pfx, const char *var, double defval );

int get_luaglobal ( lua_State*, const std::string&, const std::string& = "" );
int get_luafield ( lua_State*, const std::string&, const std::string& = "" );

#endif /* ! LUA_STACK_H */
