/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *       Free Software Foundation, Inc.
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <libgen.h>

#include <tracefctxx/TraceFct.h>
#include <suplib/file.h>

#include "main.h"
#include "common.h"
#include "gparam.h"
#include "photon.h"

#include "config.h"

#include "usage.h"

int
main(int argc, char *argv[])
{
  argv[0] = basename( argv[0] );
  TraceFct( argv[0], 0, -1 );

  switch ( gparam(argc, argv) )
  {
  case 1:
    fprintf(stderr, "%s %s\n", argv[0], VERSION);
    return EXIT_SUCCESS;

  case 2:
    usage();
    return EXIT_SUCCESS;
  }

  try {

      /* start up the log file, if requested */
      if ( sys.logfile[0] )
	  TraceFct::open( sys.logfile );

      /* initialize system */
      system_init( );

      /* generate photons */
      photon_loop( );

      /* cleanup the generators */
      system_cleanup( );

      /* close the log file */
      TraceFct::close( );

      /* clean up parameter cruft */
      cleanup_params( );
      return EXIT_SUCCESS;

  }

  catch( Exception& e ) {

      TraceFct::die( e );

  }

  return EXIT_FAILURE;
}
