#ifndef POS_GEN_H
#define POS_GEN_H

typedef struct PositionGenerator PositionGenerator;

#include "source.h"
#include "events.h"

typedef struct
{
  double	max_extent;		/* maximum radial extent of the source
					   from the optical axis [radians] */

  double	factor;			/* the extent factor by which the
  					   effective flux is multiplied */

  double	z;			/* z coord of source */

  void *	data;			/* generator specific data */
} PositionGeneratorData;

typedef int(*PosGenInit)(PositionGeneratorData*);
typedef int(*PosGenGenerate)(void*, DVector3*, double*);
typedef void(*PosGenCleanup)(void*);

typedef struct
{
    const char *	    name;	/* the name of the flux generator */
    PosGenInit     init;
    PosGenGenerate generate;
    PosGenCleanup  cleanup;
} PositionGeneratorSpec;

struct PositionGenerator
{
  Source	*	source;		/* the parent source of the
					   generator */
  PositionGeneratorSpec spec;
  PositionGeneratorData data;
  char *	name;

};


void new_pos_gen(PositionGeneratorSpec *spec, const char *name, void *data);
int  delete_pos_gen(void *data);
void init_pos_gen(PositionGenerator *gen);
void cleanup_extent_event(Event *event);

#endif /* ! POS_GEN_H */
