#ifndef SOURCE_H
#define SOURCE_H

#include <vector>

#include <lua.hpp>

typedef struct Source Source;

#include "pos_gen.h"
#include "dir_gen.h"

struct Source
{
  char *	name;			/* the name of the source */

  size_t	n_gen;			/* the number of flux generators */
  SLinkList 	flux_gen;		/* array of flux generators */

  SLinkList 	phot_valid;		/* a list of generators with valid
					   photons, sorted by arrival time */
  SLinkList 	phot_invalid;		/* a list of generators with invalid
					   photons */
  SLinkList 	phot_temp;		/* temporary list to ease swapping
  					   valid and invalid lists */

  PositionGenerator *	pos_gen;
  DirectionGenerator *	dir_gen;

  double	theta;			/* off axis angle of source */


  double	dist;			/* distance to source [mm] */

  double	time;			/* clock */

  SLinkList 	extent_events;		/* list of extent generator events */
  size_t	extent_iter_lim;	/* maximum number of iterations to
  					   satisfy extent events */
  size_t	extent_iter_warn;	/* number of iterations
  					   between extent warnings */
};


void sources_setup( void );
void sources_init( void );
void sources_cleanup( void );

Source* new_source(lua_State *L, char *name);

#endif /* ! SOURCE_H */
