/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2010 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

#ifndef SOURCE_POS_H
#define SOURCE_POS_H

#include <vector>
#include <string>

#include <lua.hpp>

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <Jitter/Common.h>

typedef enum {

    SOURCE_POS_UNKNOWN = 0,
    SOURCE_POS_ANGULAR,
    SOURCE_POS_LINEAR

} SourcePosType;


typedef struct {

    Eigen::Quaterniond *pos;

    double z;
    double theta;
    double phi;

} SourcePosAngular;

typedef struct {

    double x, y, z;

} SourcePosLinear;

typedef union SourcePos {

    SourcePosAngular angular;
    SourcePosLinear linear;

} SourcePos;

double get_src_pos( lua_State*, const char*, SourcePosType*, SourcePos * );

void  get_axes( lua_State*, const std::string&, const char*, std::vector<Eigen::Vector3d>& );

int get_axis( lua_State *L, const std::string& err_pfx, double* coords  );
int get_axis( lua_State *L, const std::string& err_pfx, const char* name, Eigen::Vector3d& axis );

Jitter::EulerAxis* get_rotation_axis( lua_State*, const std::string&, lua_Integer );

int get_euler_axis( lua_State* L, const std::string& err_pfx, const char* name, double& angle, Eigen::Vector3d &axis );

void get_quat_from_angle_axes( lua_State*, const std::string&, Eigen::Quaterniond&);

int get_quat_from_mat3d( lua_State* L, const std::string&, Eigen::Quaterniond& );


#endif /* ! SOURCE_POS_H */
