/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of raygen
 *
 * raygen is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * raygen is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <math.h>
#include <string.h>
#include <limits.h>
#include <stdlib.h>

#include <lua.hpp>
#include <lauxlib.h>

#include "main.h"
#include "common.h"
#include "lua_stack.h"

#include "source.h"

static int start( lua_State *L );
static int finish( lua_State *L );

luaL_Reg src_start = { "start", start };
luaL_Reg src_finish   = { "finish",  finish };


static int
start( lua_State *L )
{
    char *name = get_lua_cstring( L, "name" );

    if ( NULL != sys.current_source )
	luaL_error( L, "source `%s': attempt to nest sources", name );

    sys.current_source = new_source( L, name );

    return 0;
}

static int
finish( lua_State *L )
{
    char *name = get_lua_cstring( L, "name" );

    if ( NULL == sys.current_source )
	luaL_error( L, "source `%s': source finish outside of source specification?",
		 name );

    if ( strcmp( sys.current_source->name, name ) )
	luaL_error( L, "attempt to finish source `%s' while creating source `%s'",
		 name, sys.current_source->name );

    /* check source to ensure that everything was specified */
    if ( 0 == sys.current_source->n_gen )
	luaL_error( L, "source `%s': no flux generator\n", sys.current_source->name );

    if ( NULL == sys.current_source->pos_gen )
	luaL_error( L, "source `%s': no position generator\n", sys.current_source->name );

    sys.current_source = NULL;
    sys.current_fgen   = NULL;

    free( name );

    return 0;
}

