#ifndef RAYGEN_CONFIG_H
#define RAYGEN_CONFIG_H

#include <lua.hpp>
#include <lauxlib.h>

#include <tracefctxx/TraceFct.h>

#include "LuaState.h"

#include "lua_stack.h"

class SysConf {

 public:

SysConf( LuaState& L, const string& config, const string& module ) : _L(L), _module(module) {

	_L.get_module_attr( config.c_str(), _module.c_str() );
	reference = luaL_ref(_L, LUA_REGISTRYINDEX );
    }

    ~SysConf( ){

	if ( _L )
	    luaL_unref(_L, LUA_REGISTRYINDEX, reference );

    }

    bool get( const string& field, double* value = NULL ) {

	bool is_valid = get_value( field );
	if ( is_valid && value )
	    *value = lua_tonumber(_L, -1 );
	lua_pop( _L, 1 );
	return is_valid;
    }

    void set( const string& field, double value ) {

	lua_rawgeti( _L, LUA_REGISTRYINDEX, reference );
	lua_pushnumber( _L, value );
	lua_setfield( _L, -2, field.c_str() );
	lua_pop( _L, 1 );
    }

    void set( const string& field, const char* value ) {

	lua_rawgeti( _L, LUA_REGISTRYINDEX, reference );
	lua_pushstring( _L, value );
	lua_setfield( _L, -2, field.c_str() );
	lua_pop( _L, 1 );
    }

    double get_double( const string& field ) {
	if ( ! get_value( field ) )
	    TraceFct::die( "error accessing %s.%s\n", _module.c_str(), field.c_str() );
	double val = lua_tonumber( _L, -1 );
	lua_pop( _L, 1 );
	return val;
    }

    bool get_bool( const string& field ) {
	if ( ! get_value( field ) )
	    TraceFct::die( "error accessing %s.%s\n", _module.c_str(), field.c_str() );
	bool val = lua_toboolean( _L, -1 );
	lua_pop( _L, -1 );
	return val;
    }

    bool get_value( const string& field ) {
	lua_rawgeti( _L, LUA_REGISTRYINDEX, reference );
	if ( get_luafield( _L, field.c_str() ) ) {
	    lua_pop( _L, 1 );
	    lua_pushnil( _L );
	}
	else {
	    lua_replace( _L, -2 );
	}
	return ! lua_isnil( _L, -1 );
    }

 private:

    LuaState& _L;
    string _module;
    int  reference;
};

#endif // ! RAYGEN_CONFIG_H
