#ifndef SYSTEM_H
#define SYSTEM_H

#include <linklist/linklist.h>

#include "types.h"
#include "ea.h"
#include "source.h"
#include "jitter_gen.h"
#include "sysconf.h"

typedef struct
{
  char *	cfgfile;		/* the lua configuration file */
  char *	cfgpars;		/* extra lua configuration commands */

  char *	output;			/* the name of the output stream */
  char *	logfile;		/* logfile */

  double	max_open_angle;		/* the maximum opening angle
  					   to back of paraboloid for sources.
  					   set in init_source. set in
					   init_ system */

  double	t_open_angle;		/* tangent of the maximum opening angle
  					   set in init_source */

  long seed1, seed2, block;		/* random number seeds and starting
					   block */

  double	limit;			/* the limit on the current run */
  LimitType	limit_type;		/* the type of limit on the current
					   run */

  double        tstart, tstop;             /* range of output times */
  double	time;			/* the time of the last photon
					   emitted */

  size_t	counts;			/* the number of photons produced */

  double	z;			/* where the rays will be left */

  SLinkList	sources;		/* a list of the sources */
  SLinkList	gen;			/* a list of flux generators, one per
  					   source, representing the next photon
  					   from that source */

  EntranceAperture *ea;			/* array of entrance apertures */
  int		n_ea;			/* number of entrance apertures */
  double	ea_area;		/* total area of entrance aperture(s)*/

  Source *	current_source;		/* the current source.  in use
  					   during the creation of sources.
  					   changed by the begin_source and
  					   end_source routines */

  FluxGenerator *current_fgen;		/* the current flux generator.  in
  					   use during the addition of flux
  					   modifiers.  changed by the
  					   begin_flux_gen and
  					   end_flux_gen routines */

  SLinkList	spare_events;		/* spare event nodes stuck here by
  					   update_photon_time when it handles
  					   an event */

  JitterGenerator jitter;		/* jitter */

  Eigen::Quaterniond roll;              /* nominal telescope roll */


  SysConf*	config;

  double node;                           /* node Z position from which
					    to measure source angle */
} System;


/* system.c */
void system_init(void);
void system_cleanup(void);

#endif /* ! SYSTEM_H */
