#ifndef UNITS_H
#define UNITS_H

#include <cstdarg>
#include <string>

#include <udunits2.h>
#include <Exception/Exception.h>

namespace raygen {

    class Units {


    public:
	Units();
	~Units();
	ut_system* system() { return _system; }
	ut_unit*   radian() { return _radian; }
	ut_unit*   meter()  { return _meter;  }
	ut_unit*   mm()     { return _mm;     }
	ut_unit*   second() { return _second; }

	static Units& sys() {
	    static Units sys;
	    return sys;
	}

	static int handle_error_message( const char* fmt, va_list ap );

	static ut_unit* parse( string str ) { return parse( str.c_str() ); }
	static ut_unit* parse( const char* str );

	static bool convert( string from_str, ut_unit* to, double& value );
	static bool convert( ut_unit* from, ut_unit* to, double& value );

	static bool convert_angular( string from_str, double& value ) {
	    return convert( from_str, sys()._radian, value );
	}

	static bool is_angular( ut_unit* unit ) {

	    return ut_are_convertible( unit, sys()._radian );
	}

	static bool is_linear( ut_unit* unit ) {

	    return ut_are_convertible( unit, sys()._mm );
	}

    private:
	ut_system* _system;
	ut_unit* _radian;
	ut_unit* _meter;
	ut_unit* _mm;
	ut_unit* _second;
	static char _errmsg[1024];
    };

}

int cvt_linear( const char *str, double *value );
int cvt_angular( const char *str, double *value );

#endif /* ! UNITS_H */
