#include <string>
#include <iostream>

#include "gtest/gtest.h"
#include "Jitter/Table.h"
#include "Jitter/LJ.h"

#include "q_utils.h"

#define ARCSEC2RAD (M_PI / (180 * 3600))

struct LJParam {

    double phase;
    double amp;
    double period;
};

double
lj( LJParam p, double time ) {

    return p.amp * sin( time * 2 * M_PI / p.period  + p.phase );

}

TEST( JitterLJ, all ) {

    Jitter::Table jtbl( 100000 );

    {
	LJParam pitch = { 0, 8 * ARCSEC2RAD, 107.1 };
	LJParam yaw   = { 0, 8 * ARCSEC2RAD, 1000.0 };

	Vector3d X( 1, 0, 0 );
	Vector3d Y( 0, 1, 0 );

	// construct a table of discrete Lissajous positions

	for ( size_t idx = 0 ; idx < jtbl.size(); idx++ ) {

	    jtbl.points[idx].time = idx;
	    jtbl.points[idx].q =
		  Quaterniond( AngleAxisd( lj( yaw, idx), Y ) )
		* Quaterniond( AngleAxisd( lj( pitch, idx), X ) );
	}
    }

    {
	Jitter::LJ::Param pitch(0, 8 * ARCSEC2RAD, 107.1);
	Jitter::LJ::Param yaw(  0, 8 * ARCSEC2RAD, 1000);

	Jitter::LJ lj( "test", pitch, yaw );

	// jitter is the cumulative rotation, dj is the delta from the last call to jitter();
	Quaterniond jitter( 1, 0, 0, 0);
	Quaterniond dj;

	for ( size_t idx = 0 ; idx < jtbl.size(); idx++ ) {

	    lj.jitter( jtbl.points[idx].time, dj );
	    jitter = jitter * dj;

	    EXPECT_TRUE( IsEqual( jtbl.points[idx].q, jitter, 1e-9) );
	}

    }
}
