#include "q_utils.h"

#include "Jitter/jutils.h"

using namespace Eigen;

// compare two quaternions for equality. the simplest method is to
// check if lhs * rhs.inverse() is 0
::testing::AssertionResult IsEqual( Quaterniond expected,
				    Quaterniond actual,
				    double error ) {

    Quaterniond q = expected * actual.inverse();

    if ( fabs(q.x()) < error &&
	 fabs(q.y()) < error &&
	 fabs(q.z()) < error &&
	 1 - fabs(q.w()) < error ) {

	return ::testing::AssertionSuccess();
    }

    else {

	return ::testing::AssertionFailure()
	    << "\nExpected: " << expected
	    << "\nActual  : " << actual << "\n";
    }
}

// compare two quaternions for inequality. the simplest method is to
// check if lhs * rhs.inverse() is !0
::testing::AssertionResult IsNotEqual( Quaterniond expected,
				       Quaterniond actual,
				       double error ) {

    Quaterniond q = expected * actual.inverse();

    if ( fabs(q.x()) > error ||
	 fabs(q.y()) > error ||
	 fabs(q.z()) > error ||
	 1 - fabs(q.w()) > error ) {

	return ::testing::AssertionSuccess();
    }

    else {

	return ::testing::AssertionFailure()
	    << "\nDidn't Expect: " << expected
	    << "\nActual       : " << actual << "\n";
    }
}

// compare two vectors for equality
::testing::AssertionResult IsEqual( Vector3d expected,
				    Vector3d actual,
				    double error ) {

    if ( fabs(expected(0) - actual(0)) < error &&
	 fabs(expected(1) - actual(1)) < error &&
	 fabs(expected(2) - actual(2)) < error ) {

	return ::testing::AssertionSuccess();
    }

    else {

	return ::testing::AssertionFailure()
	    << "\nExpected: " << expected
	    << "\nactual  : " << actual << "\n";
    }
}
