#include <iostream>

#include "gtest/gtest.h"

#include "SimpleFits.h"

namespace {

    class SFTTest : public ::testing::Test {

    protected:
	virtual void SetUp() {
	    fits = new SimpleFits::Table( JITTER_TABLE );
	    data = NULL;
	}

	virtual void TearDown() {

	    delete fits;

	    if ( data )
		delete[] data;

	}

	SimpleFits::Table* fits;
	double *data;

    };


    TEST_F( SFTTest, num_rows ) {

	ASSERT_EQ( 692, fits->num_rows() );
    }


    TEST_F( SFTTest, colum_info ) {

	SimpleFits::Table::Column q_att = fits->column("quaterion", "q_att");

	ASSERT_EQ( 14, q_att.colnum );

	EXPECT_EQ( TDOUBLE, q_att.typecode );
	EXPECT_EQ( 4, q_att.repeat   );
    }

    TEST_F( SFTTest, read_col_double ) {

	SimpleFits::Table::Column q_att = fits->column("quaterion", "q_att");

	{
	    LONGLONG nread = fits->read_col_double( data,
						    q_att,
						    (LONGLONG) 1,
						    fits->num_rows()
						    );
	    ASSERT_EQ( fits->num_rows(), nread );
	}

	{
	    LONGLONG row = 1;
	    LONGLONG nread;

	    while( ( nread = fits->read_col_double( data, q_att, row, 10 ) ) )
		row += nread;

	    ASSERT_EQ( fits->num_rows() + 1, row );

	}


    }

    TEST_F( SFTTest, read_keyword_double_ok ) {

	string keyword = "RA_NOM";

	keyword = "RA_NOM";

	EXPECT_NO_THROW( fits->read_keyword_double( keyword) );

    }

    TEST_F( SFTTest, read_keyword_double_not_ok ) {

	string keyword = "NON-EXISTENT_KEYWORD";

	EXPECT_THROW( fits->read_keyword_double( keyword), Exception );

    }

    TEST_F( SFTTest, read_keyword_ok ) {

	string keyword = "RA_NOM";

	EXPECT_NO_THROW( fits->read_keyword( keyword) );

    }

    TEST_F( SFTTest, exists_column_yes ) {

	ASSERT_TRUE( fits->exists_column( "dy_err" ) );

    }

    TEST_F( SFTTest, exists_column_no ) {

	ASSERT_FALSE( fits->exists_column( "nope, no column" ) );

    }


} // namespace
